/*
	TrayIconCtlApplet
	Copyright (C) 2004 ONO, Masayuki <mono101@skill-engineering.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "trayiconimpl.h"
#include <syslog.h>

#define DBG_LOG( msg ) syslog( LOG_LOCAL4 | LOG_DEBUG, "%s", msg )

static const char* _version_ =
#include "./version.txt"
;
// popup-menu item id
static const int TIC_MI_1 = 1;
static const int TIC_MI_2 = 2;
static const int TIC_MI_3 = 3;
static const int TIC_MI_4 = 4;

TRAYICONDATA::TRAYICONDATA()
{
}

void TRAYICONDATA::init()
{
	clear();
	QString sKeyPrefix("ICON");
	Config cfg("TrayIconCtl");
	for ( int nCnt = 0; nCnt < nIconMax; nCnt ++ )
	{
		cfg.setGroup( QString( "%1%2" ).arg( sKeyPrefix ).arg( nCnt + 1 ) );
		QString name = cfg.readEntry( "Name" );
		if ( ! name.isEmpty() )
		{
			add( name, cfg.readEntry( "Path" ) );
		}
	}
}

void TRAYICONDATA::save()
{
	QString sKeyPrefix("ICON");
	Config cfg("TrayIconCtl");
	for ( int nCnt = 0; nCnt < nIconMax; nCnt ++ )
	{
		cfg.setGroup( QString( "%1%2" ).arg( sKeyPrefix ).arg( nCnt + 1 ) );
		cfg.writeEntry( "Name", m_Icon[ nCnt ].name );
		cfg.writeEntry( "Path", m_Icon[ nCnt ].path );
	}
}

void TRAYICONDATA::add( const QString& name, const QString& path )
{
	if ( !name.isEmpty() )
	{
		for ( int nCnt = 0; nCnt < nIconMax; nCnt++ )
		{
			if ( m_Icon[ nCnt ].name == name )
			{
				return;
			}
		}
		for ( int nCnt = 0; nCnt < nIconMax; nCnt++ )
		{
			if ( m_Icon[ nCnt ].name.isEmpty() )
			{
				m_Icon[ nCnt ].name = name;
				m_Icon[ nCnt ].path = path;
				m_Icon[ nCnt ].pix.load( path );
				m_Icon[ nCnt ].p_id = -1;
				m_Icon[ nCnt ].vanishtime = QDateTime();
				break;
			}
		}
	}
}

void TRAYICONDATA::modify( const QString& name, const QString& path )
{
	if ( !name.isEmpty() )
	{
		for ( int nCnt = 0; nCnt < nIconMax; nCnt++ )
		{
			if ( m_Icon[ nCnt ].name == name )
			{
				if ( m_Icon[ nCnt ].path != path )
				{
					// change icon image
					m_Icon[ nCnt ].path = path;
					m_Icon[ nCnt ].pix.load( path );
				}
				else
				{
					// clear timers
					m_Icon[ nCnt ].p_id = -1;
					m_Icon[ nCnt ].vanishtime = QDateTime();
				}
				break;
			}
		}
	}
}

void TRAYICONDATA::setRemoveWith( const QString& name, const int p_id )
{
	if ( !name.isEmpty() )
	{
		for ( int nCnt = 0; nCnt < nIconMax; nCnt++ )
		{
			if ( m_Icon[ nCnt ].name == name )
			{
				m_Icon[ nCnt ].p_id = p_id;
				break;
			}
		}
	}
}

void TRAYICONDATA::setRemoveAfter( const QString& name, const int sec )
{
	if ( !name.isEmpty() )
	{
		for ( int nCnt = 0; nCnt < nIconMax; nCnt++ )
		{
			if ( m_Icon[ nCnt ].name == name )
			{
				m_Icon[ nCnt ].vanishtime = QDateTime::currentDateTime().addSecs( sec );
#if 0
				DBG_LOG( QString("setRemoveAfter() > %1 %2").arg( name.latin1() )
							.arg( m_Icon[ nCnt ].vanishtime.toString() ).latin1() );
#endif
				break;
			}
		}
	}
}

bool TRAYICONDATA::sweep()
{
	bool bRet = FALSE;
	for ( int nCnt = 0; nCnt < nIconMax; nCnt++ )
	{
		if ( m_Icon[ nCnt ].p_id != -1 )
		{
			QString pidPath;
			pidPath = QString( "/proc/%1" ).arg( m_Icon[ nCnt ].p_id );
			if ( ! QFile::exists( pidPath ) )
			{
				remove( m_Icon[ nCnt ].name );
				bRet = TRUE;
			}
		}
	}
	return bRet;
}

bool TRAYICONDATA::sweep2()
{
	bool bRet = FALSE;
	for ( int nCnt = 0; nCnt < nIconMax; nCnt++ )
	{
		if ( ! m_Icon[ nCnt ].vanishtime.isNull()
			&& m_Icon[ nCnt ].vanishtime <= QDateTime::currentDateTime() )
		{
#if 0
			DBG_LOG( QString("sweep2() > %1 %2").arg( m_Icon[ nCnt ].name )
							.arg( m_Icon[ nCnt ].vanishtime.toString() ).latin1() );
			DBG_LOG( QDateTime::currentDateTime().toString().latin1() );
#endif
			remove( m_Icon[ nCnt ].name );
			bRet = TRUE;
		}
	}
	return bRet;
}

void TRAYICONDATA::remove( const QString& name )
{
	if ( !name.isEmpty() )
	{
		for ( int nCnt = 0; nCnt < nIconMax; nCnt++ )
		{
			if ( m_Icon[ nCnt ].name == name )
			{
				m_Icon[ nCnt ].name = "";
				m_Icon[ nCnt ].path = "";
				m_Icon[ nCnt ].pix = QPixmap();
				m_Icon[ nCnt ].p_id = -1;
				m_Icon[ nCnt ].vanishtime = QDateTime();
				break;
			}
		}
	}
}

void TRAYICONDATA::clear()
{
	for ( int nCnt = 0; nCnt < nIconMax; nCnt++ )
	{
		m_Icon[ nCnt ].name = "";
		m_Icon[ nCnt ].path = "";
		m_Icon[ nCnt ].pix = QPixmap();
		m_Icon[ nCnt ].p_id = -1;
		m_Icon[ nCnt ].vanishtime = QDateTime();
	}
}

QSize TRAYICONDATA::size()
{
	QSize rel( 0, 0 );
	for ( int nCnt = 0; nCnt < nIconMax; nCnt++ )
	{
		if ( ! m_Icon[ nCnt ].name.isEmpty() )
		{
			rel.setWidth( rel.width() + m_Icon[ nCnt ].pix.width() );
			if ( rel.height() < m_Icon[ nCnt ].pix.height() )
			{
				rel.setHeight( m_Icon[ nCnt ].pix.height() );
			}
		}
	}
	return rel;
}

void TRAYICONDATA::draw( QPainter* p )
{
	int x = 0;
	for ( int nCnt = 0; nCnt < nIconMax; nCnt ++ )
	{
		if ( ! m_Icon[ nCnt ].name.isEmpty() )
		{
			p->drawPixmap( x, 0, m_Icon[ nCnt ].pix );
			x += m_Icon[ nCnt ].pix.width();
		}
	}
}

CTrayIconImpl::CTrayIconImpl( QWidget* parent )
	: QWidget( parent, NULL )
{
	Config cfg("TrayIconCtl");
	cfg.setGroup("MISC");
	m_nInterval = cfg.readNumEntry( "SweepInterval", 30 );

	m_pChannel = new QCopChannel( "QPE/TrayIconCtl", this );
	connect( m_pChannel, SIGNAL( received( const QCString&, const QByteArray& ) ),
			this, SLOT( receiveMessage( const QCString&, const QByteArray& ) ) );

	m_pSweeper = new QTimer( this, "Sweeper" );
	connect( m_pSweeper, SIGNAL( timeout() ), this, SLOT( sweepTimer() ) );
	m_pSweeper->start( m_nInterval * 1000, FALSE );
	
	m_rData.init();
	setFixedSize( m_rData.size() );
}

CTrayIconImpl::~CTrayIconImpl()
{
	m_pSweeper->stop();
	disconnect( m_pSweeper, SIGNAL( timeout() ), this, SLOT( sweepTimer() ) );
	delete m_pSweeper;
	
	disconnect ( m_pChannel, SIGNAL( received( const QCString&, const QByteArray& ) ),
			this, SLOT( receiveMessage( const QCString&, const QByteArray& ) ) );
	delete m_pChannel;
}

void CTrayIconImpl::receiveMessage( const QCString& msg, const QByteArray& data )
{
	QDataStream stream( data, IO_ReadOnly );

	if ( msg == "add(QString,QString)" ) // name, path-pixmap
	{
		QString name,path;
		stream >> name >> path;
		m_rData.add( name, path );
		setFixedSize( m_rData.size() );
		repaint();
	}
	else if ( msg == "modify(QString,QString)") // name, path-pixmap
	{
		QString name,path;
		stream >> name >> path;
		m_rData.modify( name, path );
		repaint();
	}
	else if ( msg == "removeWith(QString,int)" ) // name, pid
	{
		QString name;
		int p_id;
		stream >> name >> p_id;
		m_rData.setRemoveWith( name, p_id );
	}
	else if ( msg == "remove(QString)") // name
	{
		QString name;
		stream >> name;
		m_rData.remove( name );
		setFixedSize( m_rData.size() );
		repaint();
	}
	else if ( msg == "removeAfter(QString,int)" ) // name, sec
	{
		QString name;
		int sec;
		stream >> name >> sec;
		m_rData.setRemoveAfter( name, sec );
		QTimer::singleShot( sec * 1000 + 1, this, SLOT( removeTimer() ) );
	}
	else if ( msg == "clear()")
	{
		m_rData.clear();
		setFixedSize( 0,0 );
		repaint();
	}
	else if ( msg == "init()")
	{
		m_rData.init();
		setFixedSize( m_rData.size() );
		repaint();
	}
	else if ( msg == "redraw()")
	{
		repaint();
	}
	else if ( msg == "save()")
	{
		m_rData.save();
	}
	else if ( msg == "enable()")
	{
	}
	else if ( msg == "disable()")
	{
	}
	else if ( msg == "version()")
	{
		version();
	}
}

void CTrayIconImpl::version()
{
	QString ver = "TrayIconCtl";
	ver.append( _version_ );
	Global::statusMessage( ver );
}

void CTrayIconImpl::mouseReleaseEvent( QMouseEvent * )
{
}

void CTrayIconImpl::menuAction( int )
{
}

void CTrayIconImpl::paintEvent( QPaintEvent * )
{
	QPainter p(this);
	m_rData.draw( &p );
}

void CTrayIconImpl::sweepTimer()
{
	if ( m_rData.sweep() )
	{
		setFixedSize( m_rData.size() );
		repaint();
	}
}

void CTrayIconImpl::removeTimer()
{
	if ( m_rData.sweep2() )
	{
		setFixedSize( m_rData.size() );
		repaint();
	}
}

